/*-------------------------------------------------------------------------
  tisdcc - compiles C-Code for TI-Calcs using the sdcc und other tools

  Copyright (C) 2005 Florian Richter <Florian_Richter@gmx.de>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef MSDOS
  #ifndef _WIN32
    #define LINUX
  #endif
#endif
#ifdef _WIN32
  #define strcasecmp(a, b) strcmp(_strlwr(a), (b))
#endif

enum types {
  ti83p_app,
  ti83p_asm,
  ti83p_ion,
  ti83p_pc
} type;

void usage(void) {
  printf("tisdcc - compiles C-Code for TI-Calcs using the sdcc und other tools\n");
  printf("usage: tisdcc calc file.c file1.o file2.o\n");
  printf("       tisdcc calc -c file.c\n");  
  printf("calc: ti83p_asm, ti83p_ion, ti83p_app ti83p_pc\n");
}

// change extention of the filename
char* chext(char* filename, const char* newext) {
  int i;
  for(i=strlen(filename)-1;  i>0; i--)
	if(filename[i]=='.') break;

  if(i>0) {
	filename[i] = '\0';
  }

  if(i==0 || newext[0]!='\0') {
	filename = strcat(filename, ".");
	filename = strcat(filename, newext);
  }
  return filename;
}

int main(int argc, char *argv[]) {
  int i;
  char* tisdccdir = getenv("TISDCC_DIR");
  char* tisdcc_cc = getenv("TISDCC_CC"); // cl or gcc
  char isCompileOnly;
  char command[255];

  if(tisdccdir == NULL || tisdccdir[0]=='\0') {
	printf("You didn't set the directory of tisdcc.\n");
	printf("Please set the environment-variable TISDCC_DIR.\n");
	exit(2);
  }
  if(tisdcc_cc == NULL) {
	tisdcc_cc = "cl";
  }

  printf("tisdcc\n");
  if(argc<2) {
	usage();
	return 1;
  }

  //for which calc?
  if(strcasecmp(argv[1], "ti83p_asm")==0) {
	type = ti83p_asm;
  } else if(strcasecmp(argv[1], "ti83p_app")==0) {
	type = ti83p_app;
  } else if(strcasecmp(argv[1], "ti83p_ion")==0) {
	type = ti83p_ion;
  } else if(strcasecmp(argv[1], "ti83p_pc")==0) {
	type = ti83p_pc;
  } else {
	printf("unkown calc: %s\n", argv[1]);
	usage();
	return 1;
  }

  // compile-only or also link?
  isCompileOnly = 0;
  for(i=2; i<argc; i++) {
	if(strcmp(argv[i], "-c")==0) {
	  isCompileOnly = 1;
	}
  }
  
  // run sdcc
  if(!isCompileOnly) {
	switch(type) {
	case ti83p_asm:
#ifdef LINUX
	  sprintf(command, "sdcc -mz80 --no-std-crt0 --code-loc 0x9DA3 \
--data-loc 0x9872 -L %s/lib -I %s/include -DTI83P \
-Wl%s/startup/ti83p_asm.o ti83p.lib", tisdccdir, tisdccdir, tisdccdir);
#else
	  sprintf(command, "sdcc -mz80 --no-std-crt0 --code-loc 0x9DA3 \
--data-loc 0x9872 -L %s\\lib -I %s\\include -DTI83P \
-Wl%s\\startup\\ti83p_asm.o ti83p.lib", tisdccdir, tisdccdir, tisdccdir);
#endif
	  break;

	case ti83p_ion:
#ifdef LINUX
	  sprintf(command, "sdcc -mz80 --no-std-crt0 --code-loc 0x9DB2 \
--data-loc 0x9872 -L %s/lib -I %s/include -DTI83P -DION \
-Wl%s/startup/ti83p_ion.o ti83p.lib", tisdccdir, tisdccdir, tisdccdir);
#else
	  sprintf(command, "sdcc -mz80 --no-std-crt0 --code-loc 0x9DB2 \
--data-loc 0x9872 -L %s\\lib -I %s\\include -DTI83P -DION \
-Wl%s\\startup\\ti83p_asm.o ti83p.lib", tisdccdir, tisdccdir, tisdccdir);
#endif
	  break;

	case ti83p_app:
#ifdef LINUX
	  sprintf(command, "sdcc -mz80 --no-std-crt0 --code-loc 0x4096 \
--data-loc 0x9872 -L %s/lib -I %s/include -DTI83P\
 -Wl%s/startup/ti83p_app.o ti83p.lib", tisdccdir, tisdccdir, tisdccdir);
#else
	  sprintf(command, "sdcc -mz80 --no-std-crt0 --code-loc 0x4096 \
--data-loc 0x9872 -L %s\\lib -I %s\\include -DTI83P\
 -Wl%s\\startup\\ti83p_app.o ti83p.lib", tisdccdir, tisdccdir, tisdccdir);
#endif
	  break;

	case ti83p_pc:
	  if(strcmp(tisdcc_cc, "gcc")==0) {
#ifdef LINUX
		chext(argv[2], "");
		sprintf(command, "gcc -I%s/include -g -Wall -DTI83P -DPC \
%s/lib/pc/ti83p.o -o %s", tisdccdir, tisdccdir, argv[2]);
#else
		chext(argv[2], "exe");
		sprintf(command, "gcc -I%s\\include -g -Wall -DTI83P -DPC \
%s\\lib\\pc\\ti83p.o -o %s", tisdccdir, tisdccdir, argv[2]);
#endif
		chext(argv[2], "c");
	  } else if(strcmp(tisdcc_cc, "cl")==0) {
		chext(argv[2], "exe");
		sprintf(command, "cl /I%s\\include /DTI83P /DPC /Fe%s \
%s\\lib\\pc\\ti83p.obj", tisdccdir, argv[2], tisdccdir);
		chext(argv[2], "c");
	  } else {
		printf("Don't understand content of TISDCC_CC\n");
		printf("Possible content: gcc or cl\n");
		return 1;
	  }
	  break;
	}
  } else {
	switch(type) {
	case ti83p_asm:
#ifdef LINUX
	  sprintf(command, "sdcc -mz80 -I %s/include -DTI83P", tisdccdir);
#else
	  sprintf(command, "sdcc -mz80 -I %s\\include -DTI83P", tisdccdir);
#endif
	  break;

	case ti83p_ion:
#ifdef LINUX
	  sprintf(command, "sdcc -mz80 -I %s/include -DTI83P -DION", tisdccdir);
#else
	  sprintf(command, "sdcc -mz80 -I %s\\include -DTI83P -DION", tisdccdir);
#endif
	  break;

	case ti83p_app:
#ifdef LINUX
	  sprintf(command, "sdcc -mz80 -I %s/include -DTI83P", tisdccdir);
#else
	  sprintf(command, "sdcc -mz80 -I %s\\include -DTI83P", tisdccdir);
#endif
	  break;

	case ti83p_pc:
	  if(strcmp(tisdcc_cc, "gcc")==0) {
#ifdef LINUX
		sprintf(command, "gcc -I%s/include -g -Wall -DTI83P -DPC", tisdccdir);
#else
		sprintf(command, "gcc -I%s\\include -g -Wall -DTI83P -DPC", tisdccdir);
#endif
	  } else if(strcmp(tisdcc_cc, "cl")==0) {
		sprintf(command, "cl /I%s\\include /DTI83P /DPC /c", tisdccdir);
		argv[2] = "";// delete "-c"
	  } else {
		printf("Don't understand content of TISDCC_CC\n");
		printf("Possible contents: gcc or cl\n");
		return 1;
	  }
	  break;
	}
  }
  // add own parameters
  for(i=2; i<argc; i++) {
	strcat(command, " ");
	strcat(command, argv[i]);
  }
  printf("tisdcc: %s\n", command);
  if(system(command)!=0) {
	printf("Error in sdcc\n");
	return 1;
  }

  if(!isCompileOnly) {
	switch(type) {
	case ti83p_asm:
	  // run srec_cat file.ihx -Intel -o file.tmp -Binary
	  strcpy(command, "srec_cat ");
	  chext(argv[2], "ihx");
	  strcat(command, argv[2]);
	  strcat(command, " -Intel -o ");
	  chext(argv[2], "tmp");
	  strcat(command, argv[2]);
	  strcat(command, " -Binary");
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in srec_cat\n");
		return 1;
	  }
	  // run cutbin file.tmp file.bin 0x9D93
#ifdef LINUX
	  sprintf(command, "%s/cutbin %s ", tisdccdir, argv[2]);
#else
	  sprintf(command, "%s\\cutbin %s ", tisdccdir, argv[2]);
#endif
	  chext(argv[2], "bin");
	  strcat(command, argv[2]);
	  strcat(command, " 0x9D93");
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in cutbin\n");
		return 1;
	  }
	  // run DevPac8X.COM file
#ifdef LINUX
	  sprintf(command, "wine %s/DevPac8X.COM ", tisdccdir);
#else
	  sprintf(command, "%s\\DevPac8X.COM ", tisdccdir);
#endif
	  chext(argv[2], "");
	  strcat(command, argv[2]);
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in DevPac8X.COM\n");
		return 1;
	  }
	  // run rm file.tmp file.bin
#ifdef LINUX
	  strcpy(command, "rm ");
#else
	  strcpy(command, "del ");
#endif
	  chext(argv[2], "tmp");
	  strcat(command, argv[2]);
	  strcat(command, " ");
	  chext(argv[2], "bin");
	  strcat(command, argv[2]);
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in rm/del\n");
		return 1;
	  }
	  break;

	case ti83p_ion:
	  // run srec_cat file.ihx -Intel -o file.tmp -Binary
	  strcpy(command, "srec_cat ");
	  chext(argv[2], "ihx");
	  strcat(command, argv[2]);
	  strcat(command, " -Intel -o ");
	  chext(argv[2], "tmp");
	  strcat(command, argv[2]);
	  strcat(command, " -Binary");
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in srec_cat\n");
		return 1;
	  }
	  // run cutbin file.tmp file.bin 0x9D93
#ifdef LINUX
	  sprintf(command, "%s/cutbin %s ", tisdccdir, argv[2]);
#else
	  sprintf(command, "%s\\cutbin %s ", tisdccdir, argv[2]);
#endif
	  chext(argv[2], "bin");
	  strcat(command, argv[2]);
	  strcat(command, " 0x9D93");
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in cutbin\n");
		return 1;
	  }
	  // run DevPac8X.COM file
#ifdef LINUX
	  sprintf(command, "wine %s/DevPac8X.COM ", tisdccdir);
#else
	  sprintf(command, "%s\\DevPac8X.COM ", tisdccdir);
#endif
	  chext(argv[2], "");
	  strcat(command, argv[2]);
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in DevPac8X.COM\n");
		return 1;
	  }
	  // run rm file.tmp file.bin
#ifdef LINUX
	  strcpy(command, "rm ");
#else
	  strcpy(command, "del ");
#endif
	  chext(argv[2], "tmp");
	  strcat(command, argv[2]);
	  strcat(command, " ");
	  chext(argv[2], "bin");
	  strcat(command, argv[2]);
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in rm/del\n");
		return 1;
	  }
	  break;

	case ti83p_app:
	  // run ihx2hex file.ihx file.hex
#ifdef LINUX
	  sprintf(command, "%s/ihx2hex ", tisdccdir);
#else
	  sprintf(command, "%s\\ihx2hex ", tisdccdir);
#endif
	  chext(argv[2], "ihx");
	  strcat(command, argv[2]);
	  strcat(command, " ");
	  chext(argv[2], "hex");
	  strcat(command, argv[2]);
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in ihx2hex\n");
		return 1;
	  }
	  // run appsign 0104.key file.hex
	  sprintf(command, "appsign %s/0104.key ", tisdccdir);
	  strcat(command, argv[2]);
	  printf("%s\n", command);
	  if(system(command)!=0) {
		printf("Error in appsign\n");
		return 1;
	  }
	  break;
	}
  }
  return 0;
}
